uniform sampler2D shadowmap;
uniform sampler2D texture;
uniform sampler2D normalmap;
uniform sampler2D heightmap;

uniform float reflection_Height;
uniform float refraction_IsRefracting;
uniform float reflection_IsReflecting;

varying float grey;
varying vec4 coord;

varying vec3 pipe_lightdir;
varying vec3 pipe_view;
varying vec2 pipe_coord0;

varying float worldHeight;

float getShadowOfFragment(vec3 shadowcoord)
{
	vec4 closestgrey = (texture2D(shadowmap,shadowcoord.xy)) + vec4(0.01,0.01,0.01,0.01)*2.1;
	if(grey < 0 || grey > closestgrey.z || shadowcoord.x <= 0 || shadowcoord.x >=1 || shadowcoord.y <= 0 || shadowcoord.y >= 1)
	{
		return 0;
	}
	else
	{
		return 1;
	}
}

float getShadowPercentage(vec3 shadowcoord,float epsilon)
{
	float sum = 0;
	float kernelsize_x = 2;
	float kernelsize_y = 2;
	for(float i=0 ; i<kernelsize_x ; i++)
	{
		for(float j=0 ; j<kernelsize_y ; ++j)
		{
			sum += getShadowOfFragment(shadowcoord+vec3(epsilon,0,0)*(1-i)+vec3(0,epsilon,0)*(1-j));
		}
	}
	return sum/(kernelsize_x*kernelsize_y);
}

vec4 getFragmentColor(vec4 frag_color , vec4 frag_diffuse , vec4 frag_specular , vec3 shadowcoord)
{
	float shadowfactor = getShadowPercentage(shadowcoord,0.003);
	return max(max(vec4(0.1,0.1,0.1,1) * frag_color , (vec4(0.9,0.9,0.9,1) * frag_diffuse) * frag_color * shadowfactor ) , vec4(1,1,1,1) * frag_specular * shadowfactor);
}

void main(void)
{
	if(reflection_IsReflecting == 1 && reflection_Height > worldHeight)
	{
		discard;
	}
	else if(refraction_IsRefracting == 1 && reflection_Height <= worldHeight)
	{
		discard;
	}

	vec3 shadowcoord = (coord.xyz/coord.w)*0.5 + vec3(0.5,0.5,0.5);
	
	vec3 frag_lightdir = normalize(pipe_lightdir);
    vec3 frag_view = normalize(pipe_view);
    vec2 frag_oldcoord0 = pipe_coord0;
    float frag_parallaxfactor = (texture2D(heightmap,frag_oldcoord0).x * 5.0 - 2.0 ) * 0.002;
    vec2 frag_coord0 = frag_oldcoord0 + frag_parallaxfactor * frag_view.xy;
    vec3 frag_normal = normalize( texture2D(normalmap,frag_coord0).xyz * 2.0 - 1.0 );
    vec4 frag_color = texture2D(texture,frag_coord0);
   
    float frag_diffuse = dot(frag_normal , frag_lightdir);
   
    vec3 frag_reflection = frag_normal * frag_diffuse * 2.0 - frag_lightdir;
    float frag_specular = pow(max(0.0 , dot(frag_view , frag_reflection)) , 64.0);
	
	gl_FragColor = getFragmentColor(frag_color,frag_diffuse,frag_specular,shadowcoord);
}
